import random
import types
from copy import copy
from lib import dynamic_objects

class BuildJob(object):
    def __init__(self, unit, building):
        self.unit = unit
        self.building = building
        self.gather = None

    def do(self):
        if self.unit.holding:
            self.gather = None
        if self.gather:
            if self.unit.holding.__class__ == "crystal":
                self.gather = None
            else:
                return self.gather.do()

        if self.building.construction == 1:
            self.unit.job = HoldPositionJob(self.unit, self.unit.map_xy, 5)
            return .1

        if self.unit.holding:
            return do_set_in_building(self.unit, self.building)
        else:
            closest = find_closest_object(self.unit, "crystal")
            if closest and self.unit.distance_to(closest.map_xy) < 30:
                return do_object_pickup(self.unit, closest)
            else:
                self.gather = GatherJob(self.unit, dynamic_objects.Crystal)
                return self.gather.do()

        # This should never happen!
        print "AI Stuck building"
        return 2

class GatherJob(object):
    def __init__(self, unit, resource_type, starting_target=None):
        self.unit = unit
        self.resource_type = resource_type
        self.target = starting_target

        self.subjob = None

        self.gathering_time_left = self.resource_type.gather_time

    def do(self):
        if self.subjob:
            if self.unit.holding or self.subjob.subjob:
                self.subjob = None
            else:
                return self.subjob.do()
        if self.unit.holding:
            # Find somewhere to drop it.
            building = None
            for b in self.unit.scene.buildings:
                if b.construction >= 1 and b.player == self.unit.player:
                    if not building or self.unit.distance_to(b.map_xy) <\
                            self.unit.distance_to(building.map_xy):
                        building = b
            if building:
                return do_set_in_building(self.unit, building)
            else:
                return do_set_down(self.unit, self.unit.map_xy)

        if not self.target:
            self.gathering_time_left = self.resource_type.gather_time
            closest_r = None
            r = None
            for r in self.unit.scene.map.static_map_objects:
                if r.name == self.resource_type.name and\
                        r.object_store.is_available(self.resource_type):
                    d = self.unit.distance_to(r.map_xy)
                    if not closest_r:
                        closest_r = (d, r)
                    else:
                        if d < closest_r[0]:
                            if r.is_space_to_gather():
                                closest_r = (d, r)
            # TODO: Better determin where and how far they should gather
            if not closest_r:# or closest_r[0] > 60:
                self.unit.job = HoldPositionJob(self.unit, self.unit.map_xy, 5)
                return .1
            self.subjob = GatherJob(self.unit, self.resource_type, closest_r[1])
            return .1

        if self.unit.holding:
            return do_set_down(self.unit, self.target.map_xy, dist=4)
        if self.unit.distance_to(self.target.map_xy) >= 2:
            return walk_to(self.unit, self.target.map_xy)
        if not self.target.object_store.is_available(self.resource_type):
            self.target = None
            return 1
        if self.gathering_time_left <= 0.001:
            self.gathering_time_left = self.resource_type.gather_time
            obj = self.target.object_store.take_object(self.resource_type)
            if obj:
                self.unit.pick_up(obj)
                return 1
            else:
                # We'll try another target
                self.target = None
                return 0.1
        else:
            self.unit.start_gathering(self.resource_type, self.target.map_xy)
            self.gathering_time_left -= 1
            return 1

class HoldPositionJob(object):
    def __init__(self, unit, target, stray_range=1, run=False):
        self.unit = unit
        self.run = run
        self.target = target
        self.stray_range = stray_range

        self.threw_last = False

    def do(self):

        # We want to walk, throw, walk, throw etc
        # unles the penguin is ticked off.
        if self.threw_last:
            self.threw_last = False
            if self.unit.distance_to(self.target) > self.stray_range:
                return walk_to(self.unit, self.target)
            else:
                # Try to position self better.
                enemies = enemies_in_range(self.unit)
                if enemies:
                    return position_for_attack(self.unit, enemies)

        if self.unit.can_throw and not self.run:
            enemies = enemies_in_range(self.unit)
            if enemies and hasattr(self.unit, 'throw_at'):
                if not isinstance(self.unit.holding, dynamic_objects.Snowball):
                    if self.unit.holding:
                        self.unit.unload(self.unit.map_xy)
                    return make_snowball(self.unit)
                if self.unit.mood["throw_walk_throw"]:
                    self.threw_last = True
                return self.unit.throw_at(random.choice(enemies))


        if self.unit.distance_to(self.target) > self.stray_range:
            return walk_to(self.unit, self.target)

        self.run = False

        #TODO: should wait a while before leaving to gather fish.
        if self.unit.unit_type == "worker":
            self.unit.job = GatherJob(self.unit, dynamic_objects.Fish)
            return 1


        if 1 == random.randint(1, 15):
            t = (self.target[0]+random.randint(-1,1),
                 self.target[1]+random.randint(-1,1))
            if t != self.target:
                self.unit.pathfinding.clear_history()
                return walk_to(self.unit, t)
        return .3

class Patrol(object):
    def __init__(self, unit, points):
        self.unit = unit
        self.points = copy(points)
        self.p_index = 0

    def do(self):
        # First check for enemies to throw at.
        if self.unit.can_throw:
            enemies = enemies_in_range(self.unit)
            if enemies and hasattr(self.unit, 'throw_at'):
                if not isinstance(self.unit.holding, dynamic_objects.Snowball):
                    if self.unit.holding:
                        self.unit.unload(self.unit.map_xy)
                    return make_snowball(self.unit)
                # TODO: Make intelligent choice
                return self.unit.throw_at(random.choice(enemies))

        # No enemies - keep patrolling! :)
        p = self.points[self.p_index]
        if self.unit.distance_to(p) > 2:
            return walk_to(self.unit, p)
        else:
            # Change to next point.
            if self.p_index < len(self.points)-1:
                self.p_index += 1
            else:
                self.p_index = 0
                self.points.reverse()
            self.unit.pathfinding.clear_history()
            return self.do()

class PickupObjectJob(object):
    def __init__(self, unit, target):
        self.unit = unit
        self.target = target

    def do(self):
        if self.target.held:
            # TODO find another nearby object of the same type?
            self.unit.job = None
            return .5
        res = do_object_pickup(self.unit, self.target)
        if self.target.held:
            self.unit.job = None
        return res

class SetdownObjectJob(object):
    """
    Job to set an object down somewhere.

    If the penguin isn't holding the object it will first go and pick it up.

    If target_xy isn't open the object will be dropped close by.

    If target_xy is a building, the object will be set in the building.
    """
    def __init__(self, unit, obj, target_xy):
        self.unit = unit
        self.target_xy = target_xy
        self.obj = obj
        self.target_building = self.unit.scene.map.tiles[target_xy].building

    def do(self):
        if not self.unit.holding is self.obj:
            if self.obj.held:
                # If some other penguin is holding the object, just give up.
                self.unit.job = None
                return .2
            return do_object_pickup(self.unit, self.obj)
        if self.target_building:
            res = do_set_in_building(self.unit, self.target_building)
            if not self.unit.holding:
                self.unit.job = None
            return res
        else:
            if self.unit.map_xy != self.target_xy:
                if self.unit.scene.map.pos_is_open(self.target_xy) or \
                        self.unit.distance_to(self.target_xy) > 2:
                    # TODO find the closest empty point to the target.
                    return walk_to(self.unit, self.target_xy)
            self.unit.job = None
            return self.unit.unload(self.target_xy)


class EnterBuildingJob(object):
    def __init__(self, unit, building):
        self.unit = unit
        self.building = building

    def do(self):
        if self.unit.in_building == self.building:
            # TODO: better decide if unit should stay inside or go out
            # (reasioning form the penguin's point of view). Perhaps based on
            # how safe it is out and the risk of this building be taken over 
            # (if the building can be taken over).
            if self.unit.unit_type == "snowballer":
                self.unit.job = HoldPositionJob(self.unit,
                    (self.unit.map_x, self.unit.map_y-3), 7)
                return self.unit.exit_building()
            return 1
        if self.unit.distance_to(self.building.map_xy) > 1:
            return walk_to(self.unit, self.building.map_xy)
        return self.unit.enter_building(self.building)


def do_object_pickup(unit, target):
    """
    do the next step to go and pick up an object.
    """
    assert(type(target.map_xy) == tuple)
    assert(not target.held)
    if unit.holding:
        return do_set_down(unit, target.map_xy, dist=2)
    # TODO discard current equipment if target is equipment.
    if target.map_xy == unit.map_xy or unit.distance_to(target.map_xy) == 1:
        return unit.pick_up(target)
    return walk_to(unit, target.map_xy)

def do_set_down(unit, map_xy, dist=0):
    """
    Set down the unit's held object within dist distance of map_xy.
    """
    if unit.distance_to(map_xy) > dist:
        return walk_to(unit, map_xy)
    # FIXME make sure that this is an empty place to set down!
    return unit.unload(unit.map_xy)

def do_set_in_building(unit, building):
    # TODO figure out *where* next to the building they should go.
    if unit.distance_to(building.map_xy) > 1:
        return walk_to(unit, building.map_xy)
    return unit.unload(object_store=building)

def walk_to(unit, target_pos):
    # TODO: avoid large groups of enemies (avoid range relative to emeny group size)
    if unit.in_building:
        return unit.exit_building()
    wt = unit.path_to(target_pos)
    if wt:
        return unit.walk_to(wt)
    else:
        return 1

def position_for_attack(unit, enemies):
    """ Try to position to better fight enemies. """

    friendly_nearby_units = []
    for u in unit.scene.map.model.unit_nodes.get_objs_around_tile(unit.map_xy):
        if u.player == unit.player:
            friendly_nearby_units.append(u)

    move_x = 0
    move_y = 0

    # Move away from allies that are too close
    for u in friendly_nearby_units:
        x1,y1 = unit.map_xy
        x2,y2 = u.map_xy
        d = ((x1-x2)**2 + (y1-y2)**2)**0.5
        if d < 3:
            if x1 < x2:
                move_x -= 1
            elif x1 > x2:
                move_x += 1
            if y1 < y2:
                move_y -= 1
            elif y1 > y2:
                move_y += 1

    # TODO: move twards trees, up hills and fall back when getting cold

    target_pos = unit.map_xy[0]+move_x, unit.map_xy[1]+move_y

    wt = unit.path_to(target_pos)
    if wt:
        return unit.walk_to(wt)
    else:
        return 1


def enemies_in_range(unit):
    # With as often as this run it should be optimised at least a little:
    # ncalls  tottime  percall  cumtime  percall
    # 671     0.402    0.001    0.662    0.001
    # (Apx 8 seconds of running time)
    game = unit.scene
    enemies = []
    # FIXME Use the unit nodes?
    for u in game.units:
        if u.player != unit.player and u.warmth > 0:
            x1,y1 = unit.map_xy
            x2,y2 = u.map_xy
            d = ((x1-x2)**2 + (y1-y2)**2)**0.5
            if d < unit.get_setting("throw_range"):
                enemies.append(u)
    return enemies

def make_snowball(unit):
    unit.start_gathering(dynamic_objects.Snowball, unit.map_xy)
    yield dynamic_objects.Snowball.gather_time*unit.get_setting("gather_speed_modifier")
    s = dynamic_objects.Snowball(unit.scene)
    unit.pick_up(s)
    yield .2

def find_fish_behavior(unit):
    # If we have a fish, eat it!
    if isinstance(unit.holding, dynamic_objects.Fish):
        unit.eat()
        return .5
    # Find the closest fish:

    closest = find_closest_object(unit, "fish")

    if closest:
        return do_object_pickup(unit, closest)
    else:
        # TODO what do we do if there aren't any fish available?
        return 1

def find_closest_object(unit, name):
    closest = (None, None)
    for o in unit.scene.dynamic_objects:
        # TODO get fish if held by friendly unit?
        if o.name == name and not o.held:
            dist = unit.distance_to(o.map_xy)
            if not closest[0] or dist < closest[0]:
                if o.object_store and o.object_store.player != unit.player:
                    continue
                closest = (dist, o)
    return closest[1]

class BehaviorTree(object):
    def __init__(self, root):
        self.root = root
        self.stack = [self.root]

    def execute(self):
        top = self.stack[-1]
        if hasattr(top, 'next'):
            try:
                res = top.next()
            except StopIteration:
                self.stack.pop()
                return self.execute()
        else:
            res = top()
        if isinstance(res, (int, float, long)):
            return res
        elif isinstance(res, types.GeneratorType):
            self.stack.append(res)
            return self.execute()
