from __future__ import division
import sys
import rabbyt
from pyglet.event import EventDispatcher
from pyglet.window import mouse, key
from pyglet.text import Label
import pyglet.font
import snowui
from lib.penguin_control import PenguinControl
from lib.brain import Patrol
from lib.building import BUILDINGS
from lib.player import Player

from lib import profiler as p

# pyglet.font.add_directory("data")


class GameControl(snowui.GUI):
    nums = {key._1:1, key._2:2, key._3:3, key._4:4, key._5:5, key._6:6,
            key._7:7, key._8:8, key._9:9}

    def __init__(self, scene):
        snowui.GUI.__init__(self)
        self.scene = scene
        self.attachments = []

    def setup(self):
        self.view = self.scene.view
        self.window = self.view.window
        self.zoom_index = 1
        self.zoom_levels = [1, 2, 5, 9]
        self.center_on = None

        self.groups = {1:set(), 2:set(), 3:set(), 4:set(), 5:set(), 6:set(),
                7:set(), 8:set(), 9:set(), }

        self.select_box_start = None
        self.select_box_end = None
        self.selected_units = set()
        self.holding = False
        self.last_set_job = 0
        self.selected_building = None

        # mode related stuff
        self.mode = None
        self.to_build = "tower2"
        self.patrol_points = []

        self.mouse_xy = None
        self.keyboard = key.KeyStateHandler()
        self.window.push_handlers(self.keyboard)

        self.gui_view = snowui.View(self.window)
        self.gui_view.x = self.gui_view.width/2
        self.gui_view.y = self.gui_view.height/2
        self.penguin_control = PenguinControl(self.gui_view, self)

        self.gui_hidden = False

        self.time_bar = rabbyt.Sprite("data/time_bar.png", x=110, y=self.gui_view.height-20)
        self.time_bar_bg = rabbyt.Sprite("data/time_bar_bg.png", x=110, y=self.gui_view.height-20)
        self.time_bar_bg.shape.width = 200

        self.score_board = snowui.Widget(texture="data/score_board.png",
            x=10, y=self.gui_view.height-190)
        #self.score_board.is_active = False
        self.players_stats = snowui.Text("", font_size=10, multiline=True,
            width=80, rx=6, ry=120)
        self.score_board.add(self.players_stats)
        self.add(self.score_board)

        self.help_messages = {"hotkeys":"Hotkeys: (press H to toggle)<br>p = patrol (select snowballers first)<br>-/+ = zoom in and out.", "patrol":"Right click to set points you want to patrol. Left click to finish. Middle click it cancel.", "build":"Towers take 5 crystal to complete. Select workers and right click on placed building to tell them to go get crystal for it. Towers can be captured by the enemy."}
        self.help_text = snowui.Text(self.help_messages["hotkeys"], width=300,
                multiline=True, font_size=11, y=80, x=10)
        self.show_help_text = True

    @property
    def scroll_speed(self):
        return 3*(8*self.view.zoom)


    def draw_gui(self):
        if self.mode == "patrol":
            for ppoint in self.patrol_points:
                s = rabbyt.Sprite("data/marker_leave.png", rgba=self.scene.player.color+(.5,))
                s.xy = self.scene.map.to_pixel(ppoint)
                s.render()
        elif self.mode == "build":
            x,y = self.scene.map.from_pixel(self.view.to_world_pixels(self.mouse_xy))
            for xx in range(BUILDINGS[self.to_build].size[0]):
                for yy in range(BUILDINGS[self.to_build].size[1]):
                    t = self.scene.map.tiles[x+xx,y+yy]
                    self.scene.map.control.highlight(t)

        assert p.push("player_stats update")
        if Player._points_has_changed:
            tx = ""
            for player in self.scene.players:
                tx += player.name+": "+str(player.points)+"<br>"
            self.players_stats.text = tx
            Player._points_has_changed = False
        assert p.pop()

        self.gui_view.set_viewport()

        if self.show_help_text:
            self.help_text.render()

        assert p.push("Render")
        self.render()
        assert p.pop()
        assert p.push("penguin_control")
        self.penguin_control.render()
        assert p.pop()

        assert p.push("time_bar")
        self.time_bar_bg.tex_shape.left = -self.scene.percent_day_done
        self.time_bar_bg.render()
        self.time_bar.render()
        assert p.pop()

#     def igloo_unload(self):
#         for u in self.selected_units:
#             if u.holding:

    def attach_mission(self, building, mapname):
        building.attach_mission(mapname)
        self.attachments.append(building)

    def on_mouse_press(self, x, y, button, modifiers):
        if self.penguin_control.collide((x,y)):
            return
        if self.selected_building:
            if self.selected_building.gui_pane.collide((x,y)):
                self.selected_building.gui_pane.on_mouse_press(x, y, button, modifiers)
                return

        x,y = pos = self.view.to_world_pixels((x,y))
        map_xy = self.scene.map.from_pixel(pos)

        if button == mouse.LEFT:
            for b in self.attachments:
                mapname = b._ctrl_attached_mission
                if rabbyt.collisions.aabb_collide_single(rabbyt.Quad((x,y,x,y)),
                        [b.sprite]):
                    panel_xy = self.view.from_world_pixels(b.sprite.xy)
                    self.scene._return = (lambda mn:lambda scene: self.scene.__class__(
                                                        scene, mn))(mapname)


        if self.mode == "patrol":
            if button == mouse.RIGHT:
                self.patrol_points.append(map_xy)
            elif button == mouse.LEFT and self.patrol_points:
                for u in self.selected_units:
                    if u.unit_type == "snowballer":
                        u.job = Patrol(u, self.patrol_points)
                self.mode = None
                self.patrol_points = []
                self.help_text.text = self.help_messages["hotkeys"]
            else:
                self.mode = None
                self.patrol_points = []
                self.help_text.text = self.help_messages["hotkeys"]
        elif self.mode == "build":
            if button == mouse.LEFT:
                try:
                    b = self.scene.map.add_building(self.to_build, map_xy)
                    b.player = self.scene.player
                    b.construction = 0.0
                except ValueError:
                    # Error feedback?
                    pass

            if button == mouse.RIGHT:
                self.mode = None
                self.help_text.text = self.help_messages["hotkeys"]
        else:
            if button == mouse.LEFT:
                if self.penguin_control.collide((x,y)):
                    return
                if self.scene.player:
                    self.start_select(pos)
            elif button == mouse.RIGHT:
                r = self.set_job(pos)
                if "gather" in r or "build" in r:
                    m = rabbyt.Sprite("data/marker_gather.png")
                elif "enter" in r or "run" in r:
                    m = rabbyt.Sprite("data/marker_run.png")
                elif r:
                    m = rabbyt.Sprite("data/marker.png")
                if r:
                    m.x = pos[0]
                    m.bottom = pos[1]-5
                    if self.scene.player:
                        m.rgb = self.scene.player.color
                    m.alpha = rabbyt.lerp(start=1, end=0, dt=.6)
                    self.scene.ui_imprents.add(m)

            self.holding = True


    def on_mouse_release(self, x, y, button, modifiers):
        if self.scene.player:
            if self.select_box_start:
                self.end_select(modifiers)



    def start_select(self, xy):
        self.select_box_start = xy

    def end_select(self, modifiers):
        ctrl_pressed = False
        shift_pressed = False
        # Shift overrules Ctrl
        if key.MOD_SHIFT & modifiers:
            shift_pressed = True
        elif key.MOD_CTRL & modifiers:
            ctrl_pressed = True
        # Forget previous selection if neither of the group selection modifiers
        # are pressed.
        if not (shift_pressed or ctrl_pressed):
            for u in self.selected_units:
                u.selected = False
            self.selected_units = set()

        if not self.select_box_end:
            self.select_box_end = self.select_box_start

        start_x, start_y = self.select_box_start
        end_x, end_y = self.select_box_end

        if end_x < start_x:
            start_x, end_x = end_x, start_x
        if end_y < start_y:
            start_y, end_y = end_y, start_y

        # First try selecting a building
        if self.selected_building:
            self.remove(self.selected_building.gui_pane)
            self.selected_building.selected = False
            self.selected_building = None
        for b in self.scene.buildings:
            if b.player != self.scene.player:
                continue
            s = b.sprite
            # Only count if entire select box is within building
            if start_x > s.left and end_x < s.right and start_y > s.bottom and end_y < s.top:
                self.selected_building = b
                b.selected = True
                self.add(b.gui_pane)
                break

        if not self.selected_building:
            # No building selected. See if we can select any units.
            # If a box is not drawn (just click) expand the range so it will
            # actually select the penguin.
            if self.select_box_end == self.select_box_start:
                start_x -= 20
                start_y -= 20
                end_x += 20
                end_y += 20

                if end_x < start_x:
                    start_x, end_x = end_x, start_x
                if end_y < start_y:
                    start_y, end_y = end_y, start_y

            for u in self.scene.units:
                if u.player != self.scene.player:
                    continue
                cx,cy = u.sprite.xy
                if cx > start_x and cx < end_x and cy > start_y and cy < end_y:
                    if ctrl_pressed and u in self.selected_units:
                        self.selected_units.remove(u)
                        u.selected = False
                    else:
                        self.selected_units.add(u)
                        u.selected = True

        self.select_box_start = self.select_box_end = None

        if self.selected_units:
            self.penguin_control.select(self.selected_units)
        else:
            self.penguin_control.deselect()


    def draw_select_box(self):
        if not self.select_box_start or not self.select_box_end: return
        box = rabbyt.Sprite()
        box.shape.width = abs(self.select_box_start[0]-self.select_box_end[0])
        box.shape.height = abs(self.select_box_start[1]-self.select_box_end[1])
        box.shape.left = min(self.select_box_start[0], self.select_box_end[0])
        box.shape.bottom = min(self.select_box_start[1], self.select_box_end[1])
        box.rgba = (0,0,.8,.3)
        box.render()

    def draw_selected(self):
        for u in self.selected_units:
            u.selected_sprite.render()

    def show_hide_gui(self):
        self.gui_hidden = not self.gui_hidden

        # Convert to a number if it's an Anim
        y = self.gui.y
        if not isinstance(y, (int,float,long)):
            y = y.get()

        if self.gui_hidden:
            self.gui.y = rabbyt.lerp(start=y, end=-110, dt=0.5)
        else:
            self.gui.y = rabbyt.lerp(start=y, end=0, dt=0.5)


    def set_job(self, (x,y), run_mode=False):
        r = set()
        if self.selected_units:
            run = False
            if self.scene.time-self.last_set_job < .3:
                r.add("run")
                run = True
            self.last_set_job = self.scene.time
            pos = self.scene.map.from_pixel((x,y))
            stray_range = min(max(1, len(self.selected_units)), 4)
            for u in self.selected_units:
                r.add(u.set_job(pos, stray_range, run))

            x += self.view.x-16
            y += self.view.y-23

        return r



    def look_at(self, x, y):
        """
        Center the screen on the given x,y coordinates.
        """
        self.view.x, self.view.y = x - self.view.scale_w//2, y - \
                self.view.scale_h//2


    def update(self):
        if not self.mouse_xy:
            return
        x,y = self.mouse_xy
        self.scroll_screen(x,y)



    def on_mouse_drag(self, x, y, dx, dy, buttons, modifiers):
        self.on_mouse_motion(x, y, dx, dy)
        if self.select_box_start:
            self.select_box_end = self.view.to_world_pixels(self.mouse_xy)

    def on_mouse_motion(self, x, y, dx, dy):
        self.mouse_xy = (x,y)

        if self.selected_building:
            if self.selected_building.gui_pane.collide((x,y)):
                self.selected_building.gui_pane.on_mouse_motion(x, y, dx, dy)

        x,y = pos = self.view.to_world_pixels((x,y))
        for b in self.attachments:
            mapname = b._ctrl_attached_mission
            if rabbyt.collisions.aabb_collide_single(
                    rabbyt.Quad((x,y,x,y)), [b.sprite]):
                if b.sprite.alpha == 1:
                    b.sprite.alpha = rabbyt.lerp(end=.5, dt=.1)
            else:
                b.sprite.alpha = rabbyt.lerp(end=1, dt=.1)



    def scroll_screen(self, x, y):
        xy = self.view.xy
        ss = self.scroll_speed
        if x < 10:
            self.view.x -= ss
            self.center_on = None
        elif x > self.window.width-10:
            self.view.x += ss
            self.center_on = None
        if y < 4:
            self.view.y -= ss
            self.center_on = None
        elif y > self.window.height - 10:
            self.view.y += ss
            self.center_on = None

        k = self.keyboard
        if k[key.RIGHT] or k[key.D]:
            self.view.x += ss
            self.center_on = None
        if k[key.LEFT] or k[key.A]:
            self.view.x -= ss
            self.center_on = None
        if k[key.UP] or k[key.W]:
            self.view.y += ss
            self.center_on = None
        if k[key.DOWN] or k[key.S]:
            self.center_on = None
            self.view.y -= ss

        if self.center_on:
            if self.center_on[0]:
                diffx = self.center_on[0]-self.view.x
                if abs(diffx) < 10:
                    self.center_on = (None,self.center_on[1])
                else:
                    self.view.x += diffx/abs(diffx) * abs(diffx)/10
            if self.center_on[1]:
                diffy = self.center_on[1]-self.view.y
                if abs(diffy) < 10:
                    self.center_on = (self.center_on[0], None)
                else:
                    self.view.y += diffy/abs(diffy) * abs(diffy)/10
        w,h = self.scene.map.size[1]*32, self.scene.map.size[0]*32
        self.view.keep_within([0, w, h, 0])

        do_scroll_x = do_scroll_y = True
        if w < self.view.scale_w:
            do_scroll_x = False
        if h < self.view.scale_h:
            do_scroll_y = False

        if not do_scroll_x:
            self.view.x = w/2
        if not do_scroll_y:
            self.view.y = h/2

        self.view.set_viewport()


    def zoom_in(self):
        if self.zoom_index == 0:
            return False
        self.zoom_index -= 1
        z = self.zoom_levels[self.zoom_index]
        self.view.zoom = rabbyt.ease(start=self.view.zoom,
                end=z, dt=0.5)
        return True

    def zoom_out(self):
        if self.zoom_index == len(self.zoom_levels)-1:
            return
        self.zoom_index += 1
        z = self.zoom_levels[self.zoom_index]
        self.view.zoom = rabbyt.ease(start=self.view.zoom,
                end=z, dt=0.5)

    def on_mouse_scroll(self, x, y, scroll_x, scroll_y):
        if scroll_y > 0:
            if self.zoom_in():
                x,y = self.view.to_world_pixels((x,y))
                self.center_on = (int(x),int(y))
        else:
            self.zoom_out()
            self.center_on = None

    def on_key_press(self, symbol, modifiers):
        if symbol == key.ESCAPE:
            self.scene._return = sys.exit
        if symbol == key.P:
            if self.selected_units:
                self.mode = "patrol"
                self.help_text.text = self.help_messages["patrol"]
        #if symbol == key.B:
        #    self.mode = "build"
        #    self.help_text.text = self.help_messages["build"]
        if symbol == key.MINUS:
            self.zoom_out()
        if symbol == key.PLUS or symbol == key.EQUAL:
            self.zoom_in()
        if symbol == key.H:
            self.show_help_text = not self.show_help_text

        # Group control
        if self.selected_units and key.MOD_CTRL & modifiers:
            if symbol in self.nums.keys():
                group_num = self.nums[symbol]

                # Clear group we are going to assign to.
                for u in self.groups[group_num]:
                    u.group = None

                for u in self.selected_units:
                    # Remove unit from any existing group it belongs to.
                    if u.group:
                        self.groups[u.group].remove(u)

                    u.group = group_num

                self.groups[group_num] = self.selected_units.copy()

        # Selecting groups
        elif symbol in self.nums.keys():
            group = self.groups[self.nums[symbol]]
            if group:
                for u in self.selected_units:
                    u.selected = False
                self.selected_units = group.copy()

                for u in self.selected_units:
                    u.selected = True
                self.look_at(u.x, u.y)



class FlashButton(snowui.Button):
    def handle_mouse_press(self, x, y, button, modifiers):
        if button == mouse.LEFT:
            self.is_hovering = False
            self.rgba = (1,1,1,0)
            self.handle_mouse_motion(x, y)
            snowui.Button.handle_mouse_press(self, x, y, button, modifiers)
