import rabbyt, os

from operator import attrgetter


class DynamicObjectStore(object):
    def __init__(self, scene):
        self.scene = scene
        self.objects = {}
        self.obj_counter = {}
        self.player = None

        for o in OBJECT_TYPES.values():
            self.objects[o] = []
            self.obj_counter[o] = 0

    def amount_changed(self, cls):
        pass

    def add_object(self, obj):
        cls = obj.__class__
        self.objects[cls].append(obj)

        self.obj_counter[cls] += 1
        self.amount_changed(cls)

    def take_object(self, cls):
        # Return False if the count is 0 or doesn't even have the key.
        if not self.is_available(cls):
            return False

        self.obj_counter[cls] -= 1
        if self.player:
            self.player.points -= cls.points
        self.amount_changed(cls)

        if self.objects.has_key(cls) and len(self.objects[cls]) > 0:
            obj = self.objects[cls].pop()
        else:
            obj = cls(self.scene)

        return obj

    def is_available(self, cls):
        if self.obj_counter.has_key(cls) and self.obj_counter[cls] > 0:
            return True
        return False

class DynamicObject(object):
    gather_time = 1
    required_tool = None

    # The number of points the player wins/loses for getting/loseing this object
    points = 0

    def __init__(self, scene):
        self.scene = scene
        self.scene.dynamic_objects.append(self)
        self.sprite = rabbyt.Sprite(os.path.join("data", self.name+".png"))

        self._map_xy = [0,0]

        self._object_store = None
        self._held = None

        self.current_node = self.scene.sprite_sets.get(*self.sprite.xy)
        self.current_node.add(self.sprite)

    def destroy(self):
        if self.held:
            self.held.holding = None
        if self in self.scene.dynamic_objects:
            self.scene.dynamic_objects.remove(self)
        if self._object_store:
            if self in self._object_store.objects:
                self._object_store.objects.remove(self)

        if self.current_node:
            self.current_node.remove(self.sprite)
        self.current_node = None

    def get_map_xy(self):
        if self.held:
            return self.held.map_xy
        elif self.object_store:
            # FIXME do we want to do something other than raise an exception
            # when object_store has no map_xy?
            return self.object_store.map_xy
        else:
            return self._map_xy

    map_x = property(lambda self:self.map_xy[0])
    map_y = property(lambda self:self.map_xy[1])

    @property
    def y(self):
        """ Used for sorting """
        return self.sprite.y+10

    def render(self):
        self.sprite.render()

    def set_on_ground(self, map_xy):
        self._held = None

        if self._object_store:
            if self in self._object_store.objects:
                self._object_store.objects.remove(self)
        self._object_store = None


        if self.name != "snowball":
            self._map_xy = tuple(map_xy)
            self.sprite.xy = self.scene.map.to_pixel(map_xy)
            current_node = self.scene.sprite_sets.get(*self.sprite.xy)
            if not (self.current_node is current_node):
                if self.current_node:
                    self.current_node.remove(self.sprite)
                current_node.add(self.sprite)
                self.current_node = current_node


    def set_in_object_store(self, object_store):
        self._held = None
        self._object_store = object_store
        object_store.add_object(self)

        if self.current_node:
            self.current_node.remove(self.sprite)
        self.current_node = None

    def set_held(self, held):
        self._held = held
        if self._object_store:
            self._object_store.take_object(self.__class__)
            self._object_store = None

        if self.current_node:
            self.current_node.remove(self.sprite)
        self.current_node = None

    held = property(attrgetter('_held'), set_held)
    object_store = property(attrgetter('_object_store'), set_in_object_store)
    map_xy = property(get_map_xy, set_on_ground)

    def set_map_xy(self, x, y):
        # This is used by sbscript
        self.map_xy = x,y

        # Is this ever called on client?
        if not networking.is_server():
            if self.current_node:
                self.current_node.remove(self.sprite)
            self.current_node = None


class Snowball(DynamicObject):
    gather_time = 1
    name = "snowball"
    points = 0


class Fish(DynamicObject):
    gather_time = 15
    name = "fish"
    points = 0

class Crystal(DynamicObject):
    gather_time = 20
    name = "crystal"
    points = 2

OBJECT_TYPES = {}
for o in [Fish, Crystal, Snowball]:
    OBJECT_TYPES[o.name] = o
