from __future__ import division
import sys
import rabbyt, snowui
from pyglet.event import EventDispatcher
from pyglet.window import mouse, key
from pyglet.text import Label
import pyglet.font
from pyglet import image

from lib import snowflakes
from lib.scene import Scene

from lib.game import GameScene

from pyglet.gl import *

class MainMenuScene(Scene):
    day_cycle_time = 60*3

    textures = {
        "snowflakes": image.ImageGrid(image.load("data/snowparticles.png"),
                        rows=2, columns=2)
    }

    def __init__(self, scene_handler):
        self.__loading = False
        self.scene_handler = scene_handler

        self.snowflakes = snowflakes.Snowflakes(self)

        self.ctrl = self.gui = snowui.GUI(texture="data/menu.png")
        self.gui.x = 100
        self.gui.on_key_press = self.on_key_press

        Scene.__init__(self)


        self.flash_panel = rabbyt.Sprite(shape=(self.view.width+200,
                self.view.height), rgba=(1,1,1,0), x=self.gui.shape.width/2+self.gui.x,
                y=self.view.height/2)

        #self.info_bar = InfoBar(self.gui)
        #self.gui.add(self.info_bar)

        t = snowui.Text("Play", ry=8, anchor_x="center")
        b1 = snowui.Button(icon=t,
            ry=200, rx=self.gui.shape.width/2-100,
            shape=(0,35,200,0), bounds="rect",
            default_color=(1,1,0,1), callback=self.select_map)
        t.rx = b1.shape.width/2
        self.gui.add(b1)

    def loop(self):
        Scene.loop(self)
        if self.__loading:
            self.view.x = self.gui.shape.width/2
            self.view.y = self.view.height/2
            rabbyt.clear((1,1,1,1))
            l = rabbyt.Sprite(texture="data/loading.png",
                    x=self.gui.shape.width/2, y=self.view.height/2)
            l.render()
        else:
            self.view.x = self.view.width/2
            self.view.y = self.view.height/2
            rabbyt.clear((199/255,217/255,228/255,1))
            self.gui.render()
            self.snowflakes.run(True)

        if self.flash_panel.alpha > 0:
            glBlendFunc(GL_SRC_ALPHA, GL_ONE)
            self.flash_panel.render()
            glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)

        self.scene_handler.window.flip()
        return self._return

    def play(self, map):
        self.flash_to(lambda sh:GameScene(sh, map))

    def select_map(self):
        for c in self.gui.children:
            self.gui.remove(c)

        for i,n in enumerate("crystalcrises koth_1 farfished".split()):
            t = snowui.Text(n, ry=8, anchor_x="center")
            b1 = snowui.Button(icon=t,
                ry=200-60*i, rx=self.gui.shape.width/2-100,
                shape=(0,35,200,0), bounds="rect",
                default_color=(1,1,0,1), callback=self.play, cb_data=n)
            t.rx = b1.shape.width/2
            self.gui.add(b1)


    def flash_to(self, scene_constructor):
        self.clock.schedule_once(self.__mid_flash, .4, scene_constructor)
        self.flash_panel.alpha = rabbyt.ease_in(start=0, end=1, dt=.4)
        #self.__mid_flash(0, scene_constructor)
    def __mid_flash(self, dt, scene_constructor):
        #self.flash_panel.alpha = 0
        self.flash_panel.alpha = rabbyt.ease_out(start=1, end=0, dt=.6)
        self.clock.schedule_once(self.__start_loading, .6, scene_constructor)
        self.__loading = True
    def __start_loading(self, dt, scene_constructor):
        self._return = scene_constructor


    def on_key_press(self, symbol, modifiers):
        if symbol == key.ESCAPE:
            sys.exit()


class InfoBar(snowui.Widget):
    def __init__(self, gui):
        self.gui = gui
        snowui.Widget.__init__(self, texture="data/info_bar.png")
        self.rx = self.gui.shape.width/2-self.shape.width/2
        self.ry = -5

        self.text = snowui.Text("Download New Version", rx=30, ry=16)
        self.add(self.text)
