from lib.map.model import MapModel
from lib.map.display import MapDisplay
from lib.map.control import MapControl

from lib import profiler as p

class MapWrapper:
    """
    The map is separated out into model, display and control for convenience
    This class ties them all back together.
    """
    def __init__(self, scene, mapname):
        self.scene = scene

        assert p.push("Model")
        self.model = MapModel(scene, mapname)
        assert p.pop()

        assert p.push("Display")
        self.display = MapDisplay(scene, self.model)
        assert p.pop()

        assert p.push("Control")
        self.control = MapControl(scene, self.display, self.model)
        assert p.pop()

        self.tiles = self.model.tiles
        self.to_pixel = self.control.to_pixel
        self.from_pixel = self.control.from_pixel
        self.size = self.model.size
        self.tile_size = self.display.tile_size
        self.static_map_objects = self.model.static_map_objects
        self.render = self.display.render
        self.use_day_night = self.model.use_day_night
        self.add_building = self.model.add_building
        self.pos_is_open = self.model.pos_is_open
        self.config = self.model.config
        self.name = self.model.name
        self.render_paper = self.display.render_paper

    def load(self):
        assert p.push("Populate map")
        self.model.load_dynamic_objects()
        self.model.load_buildings()
        assert p.pop()

        assert p.push("Terrain")
        self.display.load_terrain()
        assert p.pop()