from __future__ import division
import rabbyt

class MapControl(object):
    """
    All functions releated to user input related to the map.
    """
    def __init__(self, scene, display, model):
        self.scene = scene
        self.model = model
        self.display = display

        self.highlight_sprite = rabbyt.Sprite(shape=(32,32),
                rgba=(0,1,0,.4))

    def highlight(self, tile):
        self.highlight_sprite.right = tile.x*self.display.tile_size+32
        self.highlight_sprite.bottom = tile.y*self.display.tile_size +\
                tile.elevation*self.display.elevation_multiplier
        self.highlight_sprite.render()

    def from_pixel(self, (x, y)):
        """
        returns the tile coordinates matched by the given pixel coordinates,
        taking elevation into account.

        The pixel coordinates are relative to the map, NOT the view.
        """
        s = self.display.tile_size
        tx,ty = (max(0, (x // s) * s // s), max(0, (y // s) * s // s))
        tx = min(max(0,tx), self.model.size[0]-1)
        ty = min(max(0,ty), self.model.size[1]-1)

        match = (tx,ty)
        for i in reversed(range(0,10)):
            try:
                t = (tx,max(0,ty-i))
                if y < self.to_pixel(t)[1]+self.display.tile_size/2:
                    return t
            except KeyError:
                pass
        return match

    def to_pixel(self, (x,y)):
        s = self.display.tile_size
        x = min(max(0,x), self.model.size[0]-1)
        y = min(max(0,y), self.model.size[1]-1)
        return x*s+s/2, y*s+s/2 +self.model.tiles[(x,y)].elevation*\
                self.display.elevation_multiplier