from __future__ import division
import sys
try:
    import json
except ImportError:
    try:
        import simplejson as json
    except ImportError:
        print "Install simplejson"
        print "(easy_install python-simplejson)"
        sys.exit(0)

import Image
import os, rabbyt

from lib import terrain
from lib.path import Path, Stream
from lib import dynamic_objects
from lib import sbscript

from lib import building

from lib import profiler as p
from lib import events

class MapModel(object):
    """
    Modle definition and storage. If there were both a client and server, they
    would share this.
    """
    TYPES = {
        "tree": {"object_type":None, "idrange":4},
        "fence": {"object_type":None, "idrange":1},
#             "rock": {"object_type":None, "idrange":4},
        "fish": {"object_type":dynamic_objects.Fish, "idrange":1},
        "crystal": {"object_type":dynamic_objects.Crystal, "idrange":1},
#             "sign": {"object_type":None, "idrange":2},
    }
    def __init__(self, scene, name):
        # TODO: Optimize. On my sys took about a total of 4 seconds no including
        # terrain loading (which is another 3 seconds)
        self.name = name
        self.scene = scene

        # Used for pathfinding. Should this be elsewhere?
        self.unit_nodes = NodeSet(7)

        self.size = (0,0)
        self.tiles = {}
        self.static_map_objects = []


        # Load map
        assert p.push("Decode settings")
        self.config = config = json.loads(open(os.path.join("data", "maps", name,
                "settings.json")).read())
        self.use_day_night = int(config["map"]["use_day_night"])
        self.day_cycle_time = int(config["map"]["day_cycle_time"])
        self.number_players = int(config["map"]["number_player"])
        assert p.pop()



        assert p.push("Load map data images")
        simage = Image.open(os.path.join("data", "maps", name,
                "static_map_objects.png"))
        self.size = simage.size
        timage = Image.open(os.path.join("data", "maps", name, "terrain.png"))
        #land_image = Image.open(os.path.join("data", "maps", name, "land_types.png"))
        assert p.pop()


        assert p.push("Load tiles")
        # Terrain
        for x in range(self.size[0]):
            for y in range(self.size[1]):
                #c = land_image.getpixel((x,y))
                #if c[0] == 255:
                ttype = "snow"
                #else:
                    #ttype = "ice"
                elevation = 0
                if timage:
                    c = timage.getpixel((x,y))
                    if not isinstance(c, int):
                        c = c[0]
                    elevation = c/255.0 * 10 #steps
                t = Tile(ttype, x, y, elevation)
                self.tiles[(x,y)] = t
        assert p.pop()

        assert p.push("Load static map objects")
        # Staic map objects (Resources etc)
        for x in range(self.size[0]):
            for y in range(self.size[1]):
                t = self.tiles[(x,y)]
                #if self.pos_is_open_check((x,y)):
                c = simage.getpixel((x,y))[:-1]
                if str(c[0]) in config["obj_key"]:
                    name = config["obj_key"][str(c[0])]
                    s = StaticMapObjectModel(scene, name, variation=c[2], tile=t)
                    self.static_map_objects.append(s)
        assert p.pop()

        assert p.push("Load script areas")
        f = open(os.path.join("data", "maps", self.name, "script_areas.txt"), "r")
        for l in f.readlines():
            if ":" not in l:
                continue
            sid, map_xy = l.split(":")
            map_xy = tuple([int(i) for i in map_xy.split(" ")])
            s = ScriptArea(int(sid), map_xy)
            scene.script_areas[int(sid)] = s
            self.tiles[map_xy].script_area = s
        f.close()
        assert p.pop()


    def load_script(self):
        self.compiler = sbscript.Compiler(self.scene)
        fp = open(os.path.join("data", "maps", self.name, "script.sbs"))
        self.compiler.process(fp)

    def load_dynamic_objects(self):
        # Dynamic objects
        f = open(os.path.join("data", "maps", self.name, "dynamic_objects.txt"), "r")
        for l in f.readlines():
            name, l = l.split(":")
            map_xy = [int(i) for i in l.split()]

            o = dynamic_objects.OBJECT_TYPES[name](self.scene)
            o.set_on_ground(map_xy)
        f.close()

    def load_buildings(self):
        f = open(os.path.join("data", "maps", self.name, "buildings.txt"), "r")
        for l in f.readlines():
            # Hack to make sure this isn't just an empty newline.
            if len(l) > 5:
                name, map_xy = l.split(":")
                map_xy = tuple([int(i) for i in map_xy.split(" ")])
                self.add_building(name, map_xy)
        f.close()


    def add_building(self, name, map_xy):
        # TODO: Take into account building name
        root_tile = self.tiles[map_xy]
        newbuilding = building.BUILDINGS[name](self.scene, root_tile.xy)

        x,y = map_xy

        tiles = []
        for xx in range(newbuilding.size[0]):
            for yy in range(newbuilding.size[1]):
                c = self.tiles[x+xx,y+yy]
                tiles.append(c)

                if c.building or c.static_map_object:
                    raise ValueError("Area not clear")
                    return

        newbuilding.tiles = tiles
        newbuilding.add_to_display()
        self.scene.buildings.append(newbuilding)
        root_tile.building_root = True
        for c in tiles:
            o = c.static_map_object
            if o:
                self.static_map_objects.remove(o)
                node = self.scene.sprite_sets_staticmo.get(*o.xy)
                if o in node:
                    node.remove(o)
                c.static_map_object = None

            c.building = newbuilding
        return newbuilding

    def pos_is_open(self, pos):
        if not self.tiles.has_key(pos): return False
        for o in self.scene.dynamic_objects:
            if o.map_xy == pos:
                return False
        for u in self.scene.map.model.unit_nodes.get_objs_around_tile(pos):
            if u.map_xy == pos:
                return False
        t = self.tiles[pos]
        if not t.building and not t.static_map_object:
            return True
        return False



    def day_cycle(self, dt):
        if  self.scene.percent_day_done < 0.6 and self.scene.percent_day_done > 0.4:
            # Noon
            for u in self.scene.units:
                if u.warmth < u.cold_point:
                    u.warmth = u.cold_point
            self.scene.flash_panel.alpha = rabbyt.chain(
                rabbyt.ease_in(start=0, end=.4, dt=1),
                rabbyt.ease_in(end=0, dt=2)
            )
        else:
            # Midnight
            for r in self.static_map_objects:
                if r.gatherable:
                    r.object_store.obj_counter[dynamic_objects.OBJECT_TYPES[r.name]] = 10




class StaticMapObjectModel:
    def __init__(self, scene, name, variation, tile):
        self.scene = scene
        self.name = name
        self.variation = variation

        if name in "fish crystal".split():
            self.gatherable = True
            self.object_store = dynamic_objects.DynamicObjectStore(scene)
            self.object_store.obj_counter[dynamic_objects.OBJECT_TYPES[name]] = 10
        else:
            self.gatherable = False

        self.tile = tile
        self.map_xy = tile.xy

        tile.static_map_object = self

    def is_space_to_gather(self):
        # TODO: This could be greatly improved :P
        x = 0
        for u in self.scene.map.model.unit_nodes.get_objs_around_tile(self.map_xy):
            if u.unit_type == "worker" and hasattr(u.job, "target") and u.job.target == self:
                x += 1
                if x == 4:
                    return False
        return True


class Tile(object):
    """ A map tile. """
    def __init__(self, type, x, y, elevation=0):
        self.x, self.y = self.xy = (x,y)
        self.type = type
        self.building = None
        self.building_root = False
        self.static_map_object = None
        self.script_area = None

        self.elevation = elevation

    def set_xy(self, xy):
        self.x,self.y = tuple(xy)
    xy = property(lambda self:(self.x,self.y), set_xy)


class ScriptArea(object):
    accessable_by_sbs = ["map_xy", "last_player_over"]

    def __init__(self, id, map_xy):
        self.map_xy = map_xy
        self.id = id
        self._last_player_over = None

    def _set_last_player_over(self, v):
        self._last_player_over = v
        # TODO: This is really really poor method...
        events.fire("change", "$area"+str(self.id)+".last_player_over")
    last_player_over = property(lambda self:self._last_player_over,
            _set_last_player_over)

class NodeSet:
    def __init__(self, num_tiles):
        self.node_size = num_tiles
        self.nodes = {}

    def find_current_node(self, tilex,tiley):
        i = (tilex//self.node_size, tiley//self.node_size)
        if not self.nodes.has_key(i):
            node = set()
            self.nodes[i] = node
            return node
        else:
            return self.nodes[i]

    def get_nodes_around_tile(self, pos):
        x = int(pos[0])//self.node_size
        y = int(pos[1])//self.node_size
        idxs = ((x, y), (x-1, y), (x+1, y),
                (x, y+1), (x-1, y+1), (x+1, y+1),
                (x, y-1), (x-1, y-1), (x+1, y-1))
        for idx in idxs:
            if self.nodes.has_key(idx):
                yield self.nodes[idx]

    def get_objs_around_tile(self, pos):
        for node in self.get_nodes_around_tile(pos):
            for obj in node:
                yield obj
 
