from pyglet.gl import *
from pyglet import image
from math import hypot
import random

from ctypes import *

from rabbyt import Animable, anim_slot, lerp

class Vertex(Structure):
    _fields_ = [
        ('u', c_float), ('v', c_float),
        ('r', c_ubyte), ('g', c_ubyte), ('b', c_ubyte), ('a', c_ubyte),
        ('x', c_float), ('y', c_float)]

def l(s,e,t):
                return (e-s)*t+s

class Path(Animable):
    accessable_by_sbs = ["start_map_xy_point","end_map_xy_point"]

    texture_offset = anim_slot(0)
    def __init__(self, points, type, width=64, min_alpha=0, max_alpha=1):
        Animable.__init__(self)
        self.type = type
        self.points = points
        self.width = width
        self.halfwidth = self.width/2.
        self.min_alpha, self.max_alpha = min_alpha, max_alpha
        self.image = image.load("data/paths/%s.png" % self.type)
        self.seed = 100
        self.array = None
        self.build()

    @property
    def start_map_xy_point(self):
        return [i/32 for i in self.points[0]]
    @property
    def end_map_xy_point(self):
        return [i/32 for i in self.points[-1]]


    def render(self):
        if not self.array: return
        glEnable(GL_TEXTURE_2D)
        glBindTexture(GL_TEXTURE_2D, self.image.texture.id)
        glPushClientAttrib(GL_CLIENT_VERTEX_ARRAY_BIT)
        glInterleavedArrays(GL_T2F_C4UB_V3F, sizeof(Vertex),
                self.array)

        if self.texture_offset:
            glMatrixMode(GL_TEXTURE)
            glPushMatrix()
            glTranslatef(0, -self.texture_offset, 0)
        glDrawArrays(GL_TRIANGLE_STRIP, 0, len(self.array))
        if self.texture_offset:
            glPopMatrix()
            glMatrixMode(GL_MODELVIEW)
        glPopClientAttrib()

    def get_sub_points(self):
        points = []

        for i in range(len(self.points)):
            alpha_range = (1,1)
            if i == 0:
                start = self.points[0]
                alpha_range = (0,1)
            else:
                start = ((self.points[i][0] + self.points[i-1][0])*.5,
                        (self.points[i][1] + self.points[i-1][1])*.5)
            if i == len(self.points)-1:
                end = self.points[-1]
                alpha_range = (1,0)
            else:
                end = ((self.points[i][0] + self.points[i+1][0])*.5,
                        (self.points[i][1] + self.points[i+1][1])*.5)
            points.append(start+(alpha_range[0],))

            mid = self.points[i]

            for t in (.2,.4,.6,.8):
                s = l(start[0], mid[0], t), l(start[1], mid[1], t)
                e = l(mid[0], end[0], t), l(mid[1], end[1], t)
                points.append((l(s[0], e[0], t), l(s[1], e[1], t),
                        l(alpha_range[0], alpha_range[1], t)))
        points.append(self.points[-1]+(0,))
        return points

    def get_segments(self):
        a = None
        for b in self.get_sub_points():
            if a is not None:
                yield a, b
            a = b

    def build(self):
        if len(self.points) < 2:
            self.array=None
            return

        r = random.Random(self.seed)

        points = self.get_sub_points()

        if not self.array or len(self.array) != len(points)*2:
            self.array = (Vertex*(len(points)*2))()

        dist = 0
        last = points[0]
        for i, p in enumerate(points):
            dist += hypot(p[0]-last[0], p[1]-last[1])
            last = p
            if i == 0:
                dx = points[i+1][0] - p[0]
                dy = points[i+1][1] - p[1]
            elif i == len(points)-1:
                dx = p[0] - points[i-1][0]
                dy = p[1] - points[i-1][1]
            else:
                dx1 = points[i+1][0] - p[0]
                dy1 = points[i+1][1] - p[1]
                dx2 = p[0] - points[i-1][0]
                dy2 = p[1] - points[i-1][1]
                # normalize both
                d = 1/(hypot(dx1, dy1)+0.00000000001)
                dx1 *= d
                dy1 *= d
                d = 1/(hypot(dx2, dy2)+0.00000000001)
                dx2 *= d
                dy2 *= d

                dx = dx1+dx2
                dy = dy1+dy2
            # normalize
            d = 1/(hypot(dx, dy)+0.000000000001)
            dx *= d
            dy *= d

            px = -dy * (self.halfwidth + (r.random()-.5)*10)
            py = dx * (self.halfwidth + (r.random()-.5)*10) * .8
            v = self.array[i*2]
            v.u, v.v = 0,dist/self.image.height
            v.r, v.g, v.b, v.a = 255,255,255,int(p[2]*l(
                        self.min_alpha, self.max_alpha, r.random())*255)
            v.x, v.y = p[0]+px, p[1]+py
            v = self.array[i*2+1]
            v.u, v.v = 1,dist/self.image.height
            v.r, v.g, v.b, v.a = 255,255,255,int(p[2]*l(
                        self.min_alpha, self.max_alpha, r.random())*255)
            v.x, v.y = p[0]-px, p[1]-py

    def get_tiles(self):
        """
        Returns a set of tile coordinates that this path covers.
        """
        tiles = set()
        s = 32
        hs = s//2
        to_tile = lambda xy: int(round(xy[0]/s)), int(round(xy[1]/s))
        for a, b in self.get_segments():
            pass
        # TODO finish writing this

class Stream(object):
    accessable_by_sbs = ["start_map_xy_point", "end_map_xy_point"]
    type = "stream"
    def __init__(self, points, width=64):
        self.points = points
        self.width = width
        self.stream_bed = Path(self.points, "stream_bed", self.width)
        self.stream1 = Path(self.points, "stream1", self.width-10, min_alpha=.7)
        self.stream1.texture_offset = lerp(0,1, dt=6, extend="extrapolate")

        self.stream2 = Path(self.points, "stream2", self.width-20, min_alpha=.7)
        self.stream2.texture_offset = lerp(0,1, dt=4, extend="extrapolate")

    @property
    def start_map_xy_point(self):
        return [i/32 for i in self.points[0]]
    @property
    def end_map_xy_point(self):
        return [i/32 for i in self.points[-1]]

    def render(self):
        self.stream_bed.render()
        self.stream1.render()
        self.stream2.render()

    def build(self):
        self.stream_bed.build()
        self.stream1.build()
        self.stream2.build()
