from random import *
import math

class Pathfinding(object):
    def __init__(self, unit):
        self.unit = unit
        self.current_node = self.unit.scene.map.model.unit_nodes.find_current_node(
                self.unit.map_x, self.unit.map_y)
        self.current_node.add(self.unit)
        # A list of the last tiles this penguin walked on. Used for pathfinding.
        self.last_tiles = []

    def clear_history(self):
        self.last_tiles = []


    def walk_to(self, map_xy):
        self.last_tiles.append(map_xy)
        self.update_node()

    def update_node(self):
        # Update self in map nodes.
        current_node = self.unit.scene.map.model.unit_nodes.find_current_node(
                self.unit.map_x, self.unit.map_y)
        if not (self.current_node is current_node):
            if self.current_node:
                self.current_node.remove(self.unit)
            current_node.add(self.unit)
            self.current_node = current_node




    def path_to(self, map_xy):
        """
        Returns the next tile to get to requested map_xy. May return False if,
        depending on the penguin's personality, the path is blocked.
        """

        # We convert both to a tuple because if one is a list an the other isn't
        # it will evaluate False even if they both contain the same thing.
        if tuple(map_xy) == tuple(self.unit.map_xy):
            # Already there.
            return False

        direction = self.next_direction_to_pos(map_xy)

        turn_right_map = {
                (0,1)  :(-1,1),
                (-1,1) :(-1,0),
                (-1,0) :(-1,-1),
                (-1,-1):(0,-1),
                (0,-1) :(1,-1),
                (1,-1) :(1,0),
                (1,0)  :(1,1),
                (1,1)  :(0,1)
                }
        turn_left_map = {
                (-1, -1): (-1, 0),
                (-1, 0): (-1, 1),
                (-1, 1): (0, 1),
                (0, -1): (-1, -1),
                (0, 1): (1, 1),
                (1, -1): (0, -1),
                (1, 0): (1, -1),
                (1, 1): (1, 0)
                }

        turn_left = turn_left_map[direction]
        turn_right = direction
        while True:
            r = self.try_direction(turn_right)
            if r == "wait":
                return False
            elif r:
                direction = turn_right
                break

            r = self.try_direction(turn_left)
            if r == "wait":
                return False
            elif r:
                direction = turn_left
                break
            turn_right = turn_right_map[turn_right]
            # Check to see if we have exahsted all posibilities.
            if turn_right == turn_left:
                self.last_tiles = []
                return False
            turn_left = turn_left_map[turn_left]
        self.unit.direction = direction
        return self.unit.map_x+direction[0], self.unit.map_y+direction[1]


    def try_direction(self, direction):
        """
        Return False if penguin should keep trying different directions,
        True if it's open and "wait" if the penguin should wait.
        """
        map_xy = self.unit.map_x+direction[0], self.unit.map_y+direction[1]

        if len(self.last_tiles) >= 30:
            self.last_tiles.pop(0)
        if map_xy in self.last_tiles:
            return False

        try:
            tile = self.unit.scene.map.model.tiles[map_xy]
        except KeyError:
            return False

        if tile.building:
            # Penguins can walk on the top row of a building.
            if not tile.building.map_xy[1] == map_xy[1]:
                return False

        #e1 = tile.elevation
        #e2 = get_game().map.tiles[self.unit.map_xy].elevation

        # The x length is always 1. We divide by more to compensate for
        # elevation scale (FIXME: This doesn't really work right)
        #angle = math.degrees(math.atan((e1-e2)/2))

        #if abs(angle) > 45:
            #return False

        #for o in data.map._dynamic_objects:
            #if o.obstruction and o.position == (self.x,self.y):
                #return False

        for u in self.unit.scene.map.model.unit_nodes.get_objs_around_tile(
                (self.unit.map_x,self.unit.map_y)):
            if u is not self.unit and not u.in_building:
                if tuple(u.map_xy) == tuple(map_xy):
                    # TODO: Make decisions on if should stand still or not.
                    if u.is_moving() and self.unit.warmth > self.unit.cold_point:
                        return "wait"
                    else:
                        return False

        #TODO: Also needs to check for objects
        if tile.static_map_object:
            return False

        return True



    def next_direction_to_pos(self, map_xy):
        x,y = map_xy
        dx = x - self.unit.map_x
        dy = y - self.unit.map_y
        # Normalize...
        d = (dx**2+dy**2)**.5
        if d == 0: raise ValueError()
        dx, dy = dx*(1/d), dy*(1/d)
        # Make it so that the to right corner is (1,1) etc.
        dx *= 2**.5
        dy *= 2**.5

        x = y = 0
        if dx == 0.0:
            x = 0
        elif dx > 0 and dx > random():
            x = 1
        elif dx < 0 and dx < -random():
            x = -1
        if dy == 0.0:
            y = 0
        elif dy > 0 and dy > random():
            y = 1
        elif dy < 0 and dy < -random():
            y = -1

        return x,y