import random
from lib.brain import *
from lib import dynamic_objects

class PlayerAi:
    gather = None
    guard_pos = None
    gather_force = 0

    def __init__(self, scene, player):
        self.scene = scene
        self.player = player

        self.unit_nodes = scene.map.model.unit_nodes

        self.mobilizations = set()

        #self.units_attacking = set()
        #self.units_guarding = set()
        #self.being_attacked = False
        #self.spite = {}

        self.workers = set()
        self.snowballers = set()

    def get_units(self):
        # TODO: Run everytime a region changes hands.
        self.snowballers = set()
        self.workers = set()
        for u in self.scene.units:
            if u.player == self.player:
                if u.unit_type == "snowballer":
                    self.snowballers.add(u)
                else:
                    self.workers.add(u)

    def get_safe_index(self, map_xy, for_player):
        i = 0
        for u in self.unit_nodes.get_objs_around_tile(map_xy):
            if u.unit_type == "snowballer" and u.warmth > u.cold_point:
                if isinstance(u.job, HoldPositionJob):
                    if u.distance_to(u.job.target) > 20:
                        continue
                if u.player == for_player:
                    i += 1
                else:
                    i -= 1
        return i

    def run(self, dt):
        self.get_units()
        # TODO: make index of areas by location and perceived importance to each player
        value_index = {}
        my_value_index = {}
        for b in self.scene.buildings:
            #FIXME take into account building type
            #if b.name == "igloo":
                #continue
            v = {}
            for p in self.scene.players:
                if b.player != p or not b.player:
                    v[p] = 1
                else:
                    # TODO: Take into account location, not just distance
                    d = abs(p.igloo.map_xy[0]-b.map_xy[0]) + abs(p.igloo.map_xy[1]-b.map_xy[1])
                    if d:
                        v[p] = min(20.0/d, .8)
                    else:
                        # This shouldn't happen
                        v[p] = .8
                if p == self.player and b.player != self.player:
                    my_value_index[b.map_xy] = v[p]
            value_index[b.map_xy] = v

        for smo in self.scene.map.model.static_map_objects:
            d = abs(self.player.igloo.map_xy[0]-smo.map_xy[0]) +\
                abs(self.player.igloo.map_xy[1]-smo.map_xy[1])
            if smo.name == "crystal":
                my_value_index[smo.map_xy] = min(30.0/d, .6)
            #if smo.name == "fish":
                #my_value_index[smo.map_xy] = min(10.0/d, .2)

        safe_i = self.get_safe_index(self.player.igloo.map_xy, self.player)
        if safe_i > 2:
            # We have extra troops.

            # TODO: check invasions

            to_attack = random.choice(my_value_index.keys())
            #for target, v in my_value_index.items():
                #if not to_attack:
                    #to_attack = target
                #else:
                    #if v > my_value_index[to_attack]:
                        #to_attack = target

            op = OpMobilize(self)
            amount = random.randint(1, min(safe_i-2, 5))
            op.do_apply(self.player.igloo.map_xy, to_attack, amount)
            self.mobilizations.add(op)

        elif safe_i < 2:
            # We need more troops back at home!
            #TODO: Get closest group?
            to_r = set()
            for op in self.mobilizations:
                safe_i += len(op.selected)
                op.return_home()
                to_r.add(op)
                if safe_i > 1:
                    break

            self.mobilizations.difference_update(to_r)

        # Send some units to gather crystal
        num_gathering_fish = 0
        for u in self.workers:
            if hasattr(u.job, "resource_type") and u.job.resource_type.name == "fish":
                num_gathering_fish += 1
        if num_gathering_fish > 5:
            while num_gathering_fish > 5:
                num_gathering_fish -= 1
                u = self.workers.pop()
                u.job = GatherJob(u, dynamic_objects.Crystal)

        # Create new units
        if self.player.igloo.obj_counter[dynamic_objects.Fish] > 14:
            self.player.igloo.create_penguin(random.choice(["snowballer", "worker"]))


class HighLevelOperation(object):
    """
    Used to identify and execute high-level operations.
    """
    def __init__(self, ai):
        self.ai = ai
        self.map = ai.scene.map

class OpMobilize(HighLevelOperation):
    def check_applies_to_area(self, map_xy, player):
        """
        Checks to see if `player` has troops in area around map_xy
        """
        pass

    def do_apply(self, source_map_xy, targert_map_xy, amount):
        """
        Move troops
        """
        self.selected = set()
        i = 0
        for u in self.ai.unit_nodes.get_objs_around_tile(source_map_xy):
            if u.unit_type == "snowballer" and u.warmth > u.cold_point:
                if u.player == self.ai.player:
                    self.selected.add(u)
                    i += 1
                    if i == amount:
                        break

        stray_range = min(max(1, len(self.selected)), 4)
        for u in self.selected:
            u.set_job(targert_map_xy, stray_range, run=False)

    def return_home(self):
        for u in self.selected:
            u.set_job(self.ai.player.igloo.map_xy, 5, run=False)


class OpGather(HighLevelOperation):
    pass


class OpWeak(HighLevelOperation):
    pass


class OpStrong(HighLevelOperation):
    pass


class OpInvade(HighLevelOperation):
    pass
