import time
import sys

HEADER = '\033[95m'
OKBLUE = '\033[94m'
OKGREEN = '\033[92m'
WARNING = '\033[93m'
FAIL = '\033[91m'
ENDC = '\033[0m'

class Profiler:
    def __init__(self):
        self.stack = []
        self.stack_time = []

        self.log = {}
        self.start_time = 0

    def push(self, name):
        if not self.start_time:
            # Start timer on first push
            self.start_time = time.time()

        if len(self.stack) > 0:
            name = self.stack[-1]+" > "+name
        self.stack.append(name)
        self.stack_time.append(time.time())
        if name not in self.log:
            self.log[name] = 0
        return True

    def pop(self):
        i = self.stack.pop(-1)
        t = self.stack_time.pop(-1)
        self.log[i] += time.time() - t
        return True

    def dump(self, dump_name):
        print "\n"+HEADER+dump_name+ENDC
        total_time = time.time()-self.start_time
        items = self.log.items()
        items.sort()
        for i,t in items:
            l = i.split(" > ")
            pct = t/(total_time)*100
            if pct > 60:
                color = FAIL
            elif pct > 20:
                color = WARNING
            elif pct > 3:
                color = OKGREEN
            else:
                color = ""
            if pct > 99.5:
                pct = ">"+str(int(pct))
            else:
                pct = str(int(pct))
            print (len(l)-1)*"    "+OKBLUE+l[-1]+":"+ENDC, "%.2f"%t, color+"("+pct+"%)"+ENDC
        self.__init__()
        return True

if "debug" in sys.argv:
    __prof = Profiler()
    push = __prof.push
    pop = __prof.pop
    dump = __prof.dump
else:
    push = dump = lambda arg:True
    pop = lambda:True