"""
Very primitive scripting language for snowballz maps.
"""

from lib import events
from lib import brain
from lib import unit
from lib.victory import VictoryTimer


class VarsDict(dict):
    def __init__(self, compiler, *args):
        self.compiler = compiler
        dict.__init__(self, *args)

    def __setitem__(self, key, y):
        #print "set: ", key, y
        #if key[1] != "p" and y and str(y.__class__) == "<class 'lib.player.Player'>":
            #raise ValueError()
        i = key.split(".")
        if len(i) > 1:
            basevar = super(VarsDict, self).__getitem__(i[0])
            for a in i[1:]:
                if a in basevar.accessable_by_sbs:
                    ovar = basevar
                    basevar = getattr(basevar, a)
                else:
                    raise ValueError(str(basevar)+" does not have attr "+a)
            setattr(ovar, a, y)
        else:
            super(VarsDict, self).__setitem__(key, y)

        events.fire("change", key)

    def __getitem__(self, i):
        if i[0] != "$":
            if i.isdigit():
                return int(i)
            else:
                try:
                    return float(i)
                except ValueError:
                    pass
            return i

        i = i.split(".")

        if len(i) == 1:
            i = i[0]
            k = None
        else:
            k = i[1:]
            i = i[0]

        try:
            var = super(VarsDict, self).__getitem__(i)
        except KeyError:
            print self.keys()
            raise KeyError("ERROR: variable "+i+" does not exists.")

        if k:
            for a in k:
                if a in var.accessable_by_sbs:
                    var = getattr(var, a)
                else:
                    raise ValueError(str(var)+" does not have attr "+a)
        return var


class Block(object):
    def __init__(self, compiler):
        self.compiler = compiler
        self.code = ""
        self.conditions = []
        self.indent = 0
        self.local_vars = VarsDict(compiler)

    def compile(self, dt=None):
        for c in self.conditions:
            if not c.check():
                return
        r = self.compiler.exec_code(self.code, local_vars=self.local_vars, block=self)
        if r == "unset":
            for c in self.conditions:
                c.disconect()
            self.conditions = []
            self.local_vars = {}
            # I don't think we can completely remove this block as that would
            # mess up the block index.
    def done(self):
        pass


class Condition:
    def __init__(self, block):
        self.ws = []
        self.block = block
        self.compiled = False
        self._event = False

    def add(self, w):
        if w[0] == "$":
            lw = "$this>"+w[1:]
            i = lw.split(".")
            if len(i) > 1:
                # We replace . with __ so we store a local copy of the attribute
                # this is necessary for checking to see if it changes.
                lw = lw.replace(".", "__")
                self.block.local_vars[lw] = self.block.compiler._vars[w]
            else:
                self.block.local_vars[lw] = self.block.compiler._vars[w]
            w = lw
        self.ws.append(w)

    def compile(self):
        __vars = []

        for a in self.ws:
            if a[0] == "&":
                self._event = a[1:]
            elif a[0] == "$":
                __vars.append(a)

        if self._event:
            if self.compiled:
                events.disconnect(self._event, self._e_check)

            def _check():
                for lv in __vars:
                    if self._event == "frozen":
                        if self.block.local_vars[lv].warmth > 0:
                            return False
                    elif self._event == "change":
                        v = lv.replace("__", ".")
                        if self.block.local_vars[lv] ==\
                                self.block.compiler._vars["$"+v.split(">")[1]]:
                            return False
                return True
            self.check = _check

            def _e_check(etarget):
                for v in __vars:
                    if self._event == "change":
                        # With the change event we are interested in the name
                        # of the variable not the value to determin if we check.
                        to_check = "$"+v.split(">")[1]
                        to_check = to_check.replace("__", ".")
                        if etarget == to_check:
                            self.block.compile()
                            return
                    elif etarget == self.block.local_vars[v]:
                        self.block.compile()
                        return

            self._e_check = _e_check
            events.connect(self._event, self._e_check)
        self.compiled = True

    def disconect(self):
        events.disconnect(self._event, self._e_check)

    def check(self):
        pass

class Compiler(object):
    def __init__(self, scene):
        self.scene = scene
        self.map = scene.map

        self.object_types = {
            "Bear": self.create_bear,
            "Worker": self.create_worker,
            "Snowballer": self.create_snowballer,
            "VictoryTimer": self.create_VictoryTimer
        }

        self.funcs = {
        }
        for k in dir(self):
            if not k.startswith("_Compiler"):
                continue
            k = k[9:]
            if k.startswith("__public__"):
                self.funcs[k[10:]] = getattr(self, "_Compiler"+k)

        self._blocks = []
        self._sections = {}


        self._vars = VarsDict(self)

        # Used strictly for when parsing.
        self._is_block = False

        self._block_index = 0

    def __public__set(self, var, val):
        try:
            self._vars[var] = self._vars[val]
        except KeyError:
            print "Warning: variable", var, "is not defined."

    def __public__attach_mission(self, obj, mapname):
        # TODO do error checking here for if map exists. Fail silently.
        self.scene.ctrl.attach_mission(self._vars[obj], self._vars[mapname])

    def __public__attach_label(self, obj, *words):
        words = [str(self._vars[w]) for w in words]
        words = " ".join(words)
        self._vars[obj].attach_label(words)

    def __public__create(self, obj_type, var):
        if self.object_types.has_key(obj_type):
            o = self.object_types[obj_type]()
            self._vars[var] = o
    def __public__remove(self, var):
        # TODO provide for if this isn't a unit we are removing.
        self._vars[var].remove()

    def create_bear(self):
        b = unit.Bear(self.scene, "bear", (0,0), player=None)
        self.scene.units.append(b)
        return b
    def create_worker(self):
        b = unit.Unit(self.scene, "worker", (0,0), player=None)
        self.scene.units.append(b)
        return b
    def create_snowballer(self):
        b = unit.Unit(self.scene, "snowballer", (0,0), player=None)
        self.scene.units.append(b)
        return b
    def create_VictoryTimer(self):
        v = VictoryTimer(self.scene)
        return v

    def __public__log(self, *words):
        words = list(words)
        for i,w in enumerate(words):
            if w[0] == "$":
                words[i] = str(self._vars[w])
        print " ".join(words)

    def __public__set_xy(self, var, *args):
        if len(args) == 2:
            if len(args) == 1:
                x,y = self._vars[args[0]]
            else:
                x,y = [self._vars[i] for i in args]
            self._vars[var].set_map_xy(int(x), int(y))

        elif len(args) == 1:
            map_xy = self._vars[args[0]]
            self._vars[var].set_map_xy(*map_xy)

    def __public__set_patrol(self, var, *places):
        points = []
        oplaces = places
        places = []
        for p in oplaces:
            p = self._vars[p]
            if isinstance(p, list):
                for po in p:
                    places.append(po)
            else:
                places.append(p)

        for p in places:
            if hasattr(p, "points"):
                # It's a path
                for xy in p.points:
                    map_xy = self.map.from_pixel(xy)
                    if self.map.tiles.has_key(map_xy):
                        points.append(map_xy)
            else:
                points.append([int(i) for i in p])
        self._vars[var].job = brain.Patrol(self._vars[var], points)

    def __public__set_patrol_reverse(self, var, *places):
        points = []
        for p in places:
            p = self._vars[p]
            if hasattr(p, "points"):
                # It's a path
                for xy in p.points:
                    map_xy = self.map.from_pixel(xy)
                    if self.map.tiles.has_key(map_xy):
                        points.append(map_xy)
            else:
                points.append([int(i) for i in p])
        points.reverse()
        self._vars[var].job = brain.Patrol(self._vars[var], points)

    def __public__set_warmth(self, unit, warmth):
        warmth = float(self._vars[warmth])
        unit = self._vars[unit]
        unit.warmth = warmth

    def __public__set_can_throw(self, unit, v):
        if v == "False":
            v = False
        else:
            v = True
        unit = self._vars[unit]
        unit.can_throw = v

    def __public__del(self, var):
        del self._vars[var]

    def __public__list(self, var, *args):
        self._vars[var] = [self._vars[i] for i in args]

    def __public__add(self, var, i):
        self._vars[var] += self._vars[i]

    def __public__subtract(self, var, i):
        self._vars[var] -= self._vars[i]

    def __public__divide(self, var, i):
        self._vars[var] /= self._vars[i]

    def __public__multiply(self, var, i):
        self._vars[var] *= self._vars[i]


    def __public__local(self, block, var):
        block.local_vars["$local>"+var[1:]] = self._vars[var]

    def __public__when(self, *args):
        self._is_block = True
        self._block_index = len(self._blocks)
        b = Block(self)
        self._blocks.append(b)


        c = Condition(b)
        b.conditions.append(c)
        for a in args:
            if a == "and":
                c.compile()
                c = Condition(b)
                b.conditions.append(c)
            else:
                c.add(a)
        c.compile()

    def __public__timer(self, t):
        self._is_block = True
        self._block_index = len(self._blocks)
        b = Block(self)
        self._blocks.append(b)

        def done():
            self.scene.clock.schedule_once(b.compile, float(self._vars[t]))

        b.done = done

    def __public__repeat(self, num):
        self._is_block = True
        self._block_index = len(self._blocks)
        b = Block(self)
        self._blocks.append(b)

        def done():
            for i in range(int(self._vars[num])):
                b.compile()

        b.done = done

    def __public__if(self, *args):
        self._is_block = True
        self._block_index = len(self._blocks)
        b = Block(self)
        self._blocks.append(b)

        def done():
            if len(args) == 3:
                e1, com, e2 = args
                if com == "==":
                    if self._vars[e1] == self._vars[e2]:
                        b.compile()
                elif com == ">=":
                    if self._vars[e1] >= self._vars[e2]:
                        b.compile()
                elif com == "<=":
                    if self._vars[e1] <= self._vars[e2]:
                        b.compile()
                elif com == "!=":
                    if self._vars[e1] != self._vars[e2]:
                        b.compile()
                elif com == "<":
                    if self._vars[e1] < self._vars[e2]:
                        b.compile()
                elif com == ">":
                    if self._vars[e1] > self._vars[e2]:
                        b.compile()
            else:
                if self._vars[args[0]]:
                    b.compile()

        b.done = done


    def __public__section(self, name):
        self._is_block = True
        self._block_index = len(self._blocks)
        b = Block(self)
        self._blocks.append(b)

        self._sections[name] = b

    def __public__run(self, name):
        self._sections[name].compile()



    def process(self, fp):
        for i,area in enumerate(self.scene.script_areas.values()):
            self._vars["$area"+str(i+1)] = area

        for i,path in enumerate(self.scene.paths):
            self._vars["$path"+str(i+1)] = path

        for i,building in enumerate(self.scene.buildings):
            self._vars["$building"+str(i+1)] = building

        for i,player in enumerate(self.scene.players):
            self._vars["$player"+str(i+1)] = player

        self.exec_code(fp.read())


    def exec_code(self, lines, local_vars=None, block=None):
        if local_vars:
            self._vars.update(local_vars)

        for l in lines.split("\n"):

            # FIXME I wouldn't think we'd have to do this. They got
            # dropped out in a nested block.
            if block and local_vars:
                self._vars.update(local_vars)


            if self._is_block:
                # We don't detect multiple levels of blocks in one pass. They
                # get found when this block is executed.
                if l[-3:] == "end":
                    indent = len(l)-3
                    if indent == self._blocks[self._block_index].indent:
                        self._is_block = False
                        b = self._blocks[self._block_index]
                        if local_vars:
                            b.local_vars.update(local_vars)
                        b.done()
                        continue
                self._blocks[self._block_index].code += "\n"+l
                continue


            is_func = False
            args = []
            indent = 0
            for i in range(len(l)):
                if l[i] == " ":
                    indent += 1
                else:
                    break
            # Get everything to the left of any comment marks.
            l = l.split("#")[0].strip()
            if l == "return":
                return "return"
            if l == "unset":
                return "unset"

            for w in l.split(" "):
                if not w:
                    continue
                if is_func:
                    args.append(w)
                else:
                    if self.funcs.has_key(w):
                        is_func = w
                    else:
                        print "SBScript ERROR:", w, "is not a function. Skipping."
                        break

            if is_func:
                if is_func == "local":
                    if block:
                        self.funcs[is_func](block, *args)
                    else:
                        print "WARNING: tried to set a local in non block. Skipping."
                else:
                    self.funcs[is_func](*args)
                if self._is_block:
                    self._blocks[self._block_index].indent = indent


        if local_vars:
            # Handling it this way will trash global variables with the same
            # name as a local one.
            for k in local_vars.keys():
                try:
                    del self._vars[k]
                except:
                    # FIXME why does this ever happen?
#                     print "Internal Warning:", k, "was not in _vars"
                    pass