from __future__ import division
from math import cos, sin, sqrt
import random
import rabbyt


class SnowParticles:
    def __init__(self, x, y, direction, numparticles):
        self.x = x
        self.y = y

        self.particles = []

        dt = 400

        self.start = rabbyt.get_time()
        self.end = self.start + dt

        for i in xrange(numparticles):
            dx, dy = direction
            dx += (random.random() -1) * 30
            dy += (random.random() -1) * 30
            tex = data.snowparticle
            s = rabbyt.Sprite(tex.texture_id, tex.get_shape_pos(),
                    tex.get_shape_tex())
            s.x = rabbyt.lerp(x, x+dx, dt=dt)
            s.y = rabbyt.lerp(y, y+dy, dt=dt)
            s.rgba = (1,1,1, rabbyt.lerp(1, 0, self.start+dt/2, self.end))
            self.particles.append(s)

    def run(self, timestep):

        if rabbyt.get_time() < self.end:
            rabbyt.render_unsorted(self.particles)
            return True
        else:
            return False


class FlyingSnowball(rabbyt.Sprite):
    def __init__(self, from_unit, target):
        self.from_unit = from_unit
        self.target = target

        game = from_unit.scene

        d = ((from_unit.map_x-target[0])**2 +\
                (from_unit.map_y-target[1])**2)**.5
        dt = self.dt = d/5

        # Add a little randomness.
        if from_unit.mood:
            # Penguins don't aim when they are moody
            if d > 8:
                r = 3
            if d > 4:
                r = 2
            if d > 2:
                r = 1
            else:
                r = 0
        else:
            if d > 8:
                r = 2
            elif d > 4:
                r = 1
            else:
                r = 0

        if r:
            t_target = [i+random.randint(-r,r) for i in self.target]
            if game.map.tiles.has_key(tuple(t_target)):
                self.target = t_target


        rabbyt.Sprite.__init__(self, "data/snowball.png")

        self.shadow = rabbyt.Sprite("data/snowball_shadow.png")

        exy = list(game.map.to_pixel(self.target))
        exy[0] += random.randint(-16, 16)
        exy[1] += random.randint(-16, 16)
        xy = rabbyt.lerp(game.map.to_pixel(self.from_unit.map_xy), exy, dt=dt)
        self.shadow.y = xy[1]
        self.y = xy[1] + rabbyt.ease_out(start=0, end=40, dt=dt/2,
                extend="reverse")
        self.shadow.x = self.x = xy[0]
        game.flying_snowballs.append(self)
        game.flying_snowballs.append(self.shadow)
        game.clock.schedule_once(self.hit, dt)


    def hit(self, dt):
        game = self.from_unit.scene
        game.flying_snowballs.remove(self)
        game.flying_snowballs.remove(self.shadow)

        s = rabbyt.Sprite("data/snowball_imprent.png")
        s.alpha = rabbyt.lerp(start=1, end=0, dt=20)
        s.xy = self.xy
        game.snowball_imprents.add(s)


        if game.map.tiles[tuple(self.target)].static_map_object:
            return
        # TODO do something sane here...
        directions = [(0,1),(-1,1),(-1,0),(-1,-1),(0,-1),(1,-1),(1,0),(1,1)]
        sides = [(self.target[0]+x, self.target[1]+y) for (x,y) in directions]
        for u in game.units:
            if tuple(u.map_xy) == tuple(self.target):
                # Direct hit
                u.last_hit_by = self.from_unit
                u.warmth -= 3
                u.chance_mood(.07)
            elif tuple(u.map_xy) in sides:
                # Inderect hit
                u.last_hit_by = self.from_unit
                u.warmth -= 1
                u.chance_mood(.01)
