import rabbyt
from random import random, randint

class Flake(rabbyt.Sprite):
    def __init__(self, scene):
        self.tn = randint(0, 3)
        tex = scene.textures["snowflakes"][self.tn].texture
        rabbyt.Sprite.__init__(self, tex)

        bounds = [[0,0],[0,0]]
        bounds[0][0] = 0
        bounds[1][0] = 0
        bounds[0][1] = scene.view.scale_w
        bounds[1][1] = scene.view.scale_h

        x = random()*scene.view.scale_w
        y = random()*scene.view.scale_h

        speed = random()*20+20
        self.life = random()*200+300

        self.lifetime = self.life/speed

        x = rabbyt.lerp(x, x, dt=self.lifetime) - scene.view.attrgetter("x")
        self.x = rabbyt.wrap(bounds[0], x)
        y = rabbyt.lerp(y, -y-self.life, dt=self.lifetime) - scene.view.attrgetter("y")
        self.y = rabbyt.wrap(bounds[1], y)

        self.scale = rabbyt.ease_in(1, .1, dt=self.lifetime)
        self.alpha = rabbyt.lerp(start=0, end=1, dt=.2)

#         r = randint(0,3)
#         s = min(random()*2+.4, 2)
#         if r == 0:
#             self.rot = rabbyt.ease_out(start=0, end=359, dt=s, extend="repeat")
#         elif r == 1:
#             self.rot = rabbyt.ease_in(start=359, end=0, dt=s, extend="repeat")
#         elif r == 2:
#             self.rot = rabbyt.lerp(start=359, end=0, dt=s, extend="repeat")
#         elif r == 3:
#             self.rot = rabbyt.lerp(start=0, end=359, dt=s, extend="repeat")


class Snowflakes(object):
    def __init__(self, scene):
        self.scene = scene
        self.flakes = set()

        self.max_flakes = 100


        self.scheduler = rabbyt.Scheduler()

    def run(self, falling):

        # Update the bounds with the view information:

        self.scheduler.pump()
        if falling:
            if len(self.flakes) < self.max_flakes:
                self.add_flake()
        rabbyt.render_unsorted(self.flakes)

    def add_flake(self):
        f = Flake(self.scene)
        self.flakes.add(f)
        self.scheduler.add(self.scene.time+f.lifetime,
                lambda: self.flakes.discard(f))