import os, Image, random
import rabbyt
import sys, os
from copy import copy
sys.path.append(".")

import rabbyt
import pyglet
from pyglet.gl import *
from pyglet.clock import Clock
from pyglet.window import mouse, key
from pyglet import font
from pyglet import image
from lib.ctrl import GameControl, FlashButton
from lib import path
#from lib.resources import create_resource, remove_resource, TYPES
from lib.dynamic_objects import OBJECT_TYPES
from snowui import Window, View
import snowui

PATH_TYPES = ["tracks","stream"]

from lib.game import GameScene
from lib.scene_handler import SceneHandler
from lib.scene import Scene
from lib.map import MapWrapper
from lib.map.model import StaticMapObjectModel, MapModel
from lib.map.display import StaticMapObjectView
from lib.building import BUILDINGS

try:
    import json
except ImportError:
    try:
        import simplejson as json
    except ImportError:
        print "Install simplejson"
        print "(easy_install python-simplejson)"
        sys.exit(0)

def create_resource(map, tile, type, rot=0, id=None):
    t = TYPES[type]
    if not id:
        id = random.choice(range(t["idrange"]))
    img = "data/"+type+str(id)+".png"
    s = StaticMapObjectModel(map.scene, type, id, tile)
    map.static_map_objects.append(s)

    o = StaticMapObjectView(map.scene, s)
    node = map.scene.sprite_sets_staticmo.get(*o.xy)
    node.add(o)



def remove_resource(map, tile):
    r = tile.static_map_object
    if r:
        tile.static_map_object = None
        map.static_map_objects.remove(r)
        map.scene.sprite_sets_staticmo.get(*r.display.xy).remove(r.display)



class ScriptArea(object):
    id_count = 1
    accessable_by_sbs = ["map_xy", "id"]

    def __init__(self, map_xy, scene, id=None):
        if not id:
            self.id = ScriptArea.id_count
            ScriptArea.id_count += 1
        else:
            self.id = id
            if id >= ScriptArea.id_count:
                ScriptArea.id_count = id+1

        self.scene = scene

        self.map_xy = map_xy

        self.sprite = rabbyt.Sprite(shape=(32,32))
        self.sprite.rgba = (0,1,0,0.3)

        self.sprite.xy = self.scene.map.to_pixel(map_xy)

        self.font = font.load(os.path.join('data', 'SerafettinCartoon.ttf'), 13)
        self.text = font.Text(self.font, str(self.id))
        self.text.x = self.sprite.x
        self.text.y = self.sprite.y
        self.text.color = (1,1,1,1)

    def render(self):
        self.sprite.render()
        self.text.draw()


class DummyCtrl(GameControl):
    def __init__(self, scene):
        snowui.GUI.__init__(self)
        self.scene = scene
        self.attachments = []


    def setup(self):
        GameControl.setup(self)

        self.gui = snowui.GUI(bounds=(1024,0, 150),
                texture="data/gui/cp.png", x=1024-150)
        self.window.push_handlers(self.gui)

        self.path_node = rabbyt.Sprite(rgba=(0,0,1,0.5))

        self.box = rabbyt.Sprite("data/gui/mapedit/box.png")

        self.place_type = "tree"

        self.script_areas = {}
        for id, map_xy in self.scene.script_areas.items():
            self.script_areas[map_xy] = ScriptArea(map_xy, self.scene, id)

        OptionButton = snowui.widget_setting(FlashButton,
                texture="data/gui/buttons/circle.png", hover_color=(.3,1,.3,1))

        self.gui.add(OptionButton(icon="data/gui/icons/save.png",
                callback=self.save, rxy=(105,15)))


        self.gui.add(OptionButton(icon="data/gui/mapedit/path_icon.png",
                callback=lambda:self.set_tool("path"), rxy=(120,80),))

        self.gui.add(OptionButton(icon="data/gui/mapedit/resource_icon.png",
                callback=lambda:self.set_tool("resource"), rxy=(100,80),))

        self.gui.add(OptionButton(icon="data/gui/mapedit/objects_icon.png",
                callback=lambda:self.set_tool("objects"), rxy=(80,80),))

        self.gui.add(OptionButton(icon="data/gui/mapedit/script_icon.png",
                callback=lambda:self.set_tool("script_area"), rxy=(60,80),))

        self.gui.add(OptionButton(icon="data/gui/mapedit/buildings_icon.png",
                callback=lambda:self.set_tool("buildings"), rxy=(120,105),))

        self.keys = {}

        Pane = snowui.widget_setting(snowui.Pan, rxy=(50,40), shape=(0,26,90,0),
                drag_button=mouse.MIDDLE, scroll_range_x=(-50,0))

        self.resource_pane = Pane()

        for (i, (n,t)) in enumerate(TYPES.items()):
            img = "data/"+n+"0.png"
            self.resource_pane.add(FlashButton(texture=img, bounds="rect",
                    callback=self.set_builder,
                    cb_data=n,
                    rxy=(26*i+10,12),
                    shape=(25,25),))
            self.keys[n] = i+1


        self.buildings_pane = Pane()

        for (i, (name, building)) in enumerate(BUILDINGS.items()):
            img = "data/"+name+".png"
            self.buildings_pane.add(FlashButton(texture=img, bounds="rect",
                    callback=self.set_builder,
                    cb_data=name,
                    rxy=(26*i+10,12),
                    shape=(25,25),))


        self.path_pane = Pane()

        for (i, p) in enumerate(PATH_TYPES):
            img = "data/gui/paths/%s.png" % p
            self.path_pane.add(FlashButton(
                    texture="data/gui/buttons/circle.png",
                    icon=img,
                    callback=self.set_builder,
                    cb_data=p,
                    rxy=(26*i,0)))



        self.objects_pane = Pane()

        for (i, (name, p)) in enumerate(OBJECT_TYPES.items()):
            try:
                img = "data/gui/objects/%s.png" % p.name
                self.objects_pane.add(FlashButton(
                        texture="data/gui/buttons/circle.png",
                        icon=img,
                        callback=self.set_builder,
                        cb_data=name,
                        rxy=(26*i,0)))
            except IOError:
                pass


        self.current_path = None
        self.selected_path_point = None

        self.current_tool = None
        self.set_tool("resource")


    def draw_gui(self):
        self.gui_view.set_viewport()
        self.gui.render()


    def set_tool(self, tool):
        if tool == self.current_tool:
            return

        if tool == "resource":
            self.gui.add(self.resource_pane)
            self.place_type = "tree"
        if self.current_tool == "resource":
            self.gui.remove(self.resource_pane)

        if tool == "path":
            self.gui.add(self.path_pane)
            self.place_type = PATH_TYPES[0]
        if self.current_tool == "path":
            self.gui.remove(self.path_pane)

        if tool == "objects":
            self.gui.add(self.objects_pane)
            self.place_type = "fish"
        if self.current_tool == "objects":
            self.gui.remove(self.objects_pane)

        if tool == "buildings":
            self.gui.add(self.buildings_pane)
            self.place_type = "igloo"
        if self.current_tool == "buildings":
            self.gui.remove(self.buildings_pane)

        self.current_tool = tool


    def save(self):
        map = self.scene.map

        # save keys.
        for n,i in self.keys.items():
            map.config["obj_key"][str(i)] = n

        # write resource data to image
        pth = os.path.join("data", "maps", map.name, "static_map_objects.png")
        i = Image.open(pth)
        for (xy, t) in map.tiles.items():
            rgb = [0,0,0,255]
            if t.static_map_object:
                rgb[0] = self.keys[t.static_map_object.name]
                rgb[2] = t.static_map_object.variation
            i.putpixel(xy, tuple(rgb))
        i.save(pth)

        # Write settings file
        fp = file(os.path.join("data", "maps", map.name, "settings.json"), "w")
        fp.write(json.dumps(map.config))

        # Paths
        f = open(os.path.join("data", "maps", map.name, "paths.txt"), "w")
        txt = ""
        for pth in self.scene.paths:
            txt += pth.type+":"
            for p in pth.points:
                txt += "%i,%i " % p
            txt += "\n"
        f.write(txt)
        f.close()

        # Objects
        f = open(os.path.join("data", "maps", map.name, "dynamic_objects.txt"), "w")
        txt = ""
        for o in self.scene.dynamic_objects:
            txt += "%s:%i %i\n" % (o.name, o.map_x, o.map_y)
        f.write(txt)
        f.close()

        f = open(os.path.join("data", "maps", map.name, "script_areas.txt"), "w")
        txt = ""
        for sa in self.script_areas.values():
            txt += str(sa.id)+":%i %i\n" % sa.map_xy
        f.write(txt)
        f.close()

        f = open(os.path.join("data", "maps", map.name, "buildings.txt"), "w")
        txt = ""
        for b in self.scene.buildings:
            txt += b.name+":%i %i\n" % b.map_xy
        f.write(txt)
        f.close()

        print "Saved!"

    def set_builder(self, name):
        self.place_type = name

    def on_mouse_press(self, x, y, button, modifiers):
        if self.gui.collide((x,y)):
            return

        if self.current_tool == "resource":
            map = self.scene.map
            pos = self.view.to_world_pixels((x,y))
            pos = map.from_pixel(pos)
            t = map.tiles[pos]

            isopen = map.pos_is_open(pos)
            if isopen and button == mouse.LEFT:
                create_resource(map, t, self.place_type)

            elif not isopen and button == mouse.RIGHT:
                remove_resource(map, t)

        elif self.current_tool == "path":
            pos = self.view.to_world_pixels((x,y))

            r = self.get_path_points_collide(pos)
            if not self.current_path and r:
                if button == mouse.LEFT:
                    # Not building path, so this is editing
                    self.selected_path_point = r

                elif button == mouse.RIGHT:
                    p,index = r
                    del p.points[index]
                    if len(p.points) < 2:
                        self.scene.paths.remove(p)
                    p.build()

                elif button == mouse.MIDDLE:
                    self.scene.paths.remove(r[0])
            else:
                if button == mouse.LEFT:
                    if not self.current_path:
                        if self.place_type == "stream":
                            self.current_path = path.Stream([pos])
                        else:
                            self.current_path = path.Path([pos], self.place_type)
                    else:
                        self.current_path.points.append(pos)
                        self.current_path.build()

                elif button == mouse.RIGHT:
                    if self.current_path:
                        # Make sure the path is long enough. If not destroy it as the
                        # user isn't left clicking to extend it but finish the path
                        # constructions.
                        if len(self.current_path.points) > 1:
                            self.scene.paths.append(self.current_path)
                        self.current_path.build()
                        self.current_path = None

        elif self.current_tool == "objects":
            map = self.scene.map
            pos = self.view.to_world_pixels((x,y))
            pos = map.from_pixel(pos)
            t = map.tiles[pos]

            if button == mouse.LEFT:
                o = OBJECT_TYPES[self.place_type](self.scene)
                o.set_on_ground(pos)

            elif button == mouse.RIGHT:
                for o in copy(self.scene.dynamic_objects):
                    if o.map_xy == pos:
                        self.scene.dynamic_objects.remove(o)
                        o.current_node.remove(o.sprite)
                        break


        elif self.current_tool == "script_area":
            map = self.scene.map
            pos = self.view.to_world_pixels((x,y))
            pos = map.from_pixel(pos)
            t = map.tiles[pos]

            if button == mouse.LEFT:
                if self.script_areas.has_key(pos):
                    return

                sa = ScriptArea(pos, self.scene)
                self.script_areas[pos] = sa

            elif button == mouse.RIGHT:
                if self.script_areas.has_key(pos):
                    del self.script_areas[pos]

        elif self.current_tool == "buildings":
            map = self.scene.map
            pos = self.view.to_world_pixels((x,y))
            pos = map.from_pixel(pos)
            t = map.tiles[pos]

            if button == mouse.LEFT:
                try:
                    map.add_building(self.place_type, pos)
                except ValueError:
                    pass

            elif button == mouse.RIGHT:
                b = t.building
                if b:
                    self.scene.buildings.remove(b)
                    b.node.remove(b)
                    t.building_root = False
                    for tile in b.tiles:
                        tile.building = None




    def get_path_points_collide(self, pos):
        for path in self.scene.paths:
            c = rabbyt.collisions.collide_single((pos[0],pos[1],20),path.points)
            if c:
                return path, path.points.index(c[0])

    def on_mouse_release(self, x, y, button, modifiers):
        self.selected_path_point = None

    def on_mouse_drag(self, x, y, dx, dy, buttons, modifiers):
        pos = self.view.to_world_pixels((x,y))
        if self.selected_path_point:
            p, index = self.selected_path_point
            p.points[index] = pos
            p.build()
        else:
            if self.current_tool in "resource script_area".split():
                self.on_mouse_press(x, y, buttons, modifiers)

    def draw_m(self):
        if not self.mouse_xy: return
        pos = self.view.to_world_pixels(self.mouse_xy)
        if self.current_tool == "resource":
            map = self.scene.map
            mpos = map.from_pixel(pos)

            if map.pos_is_open(mpos):
                self.box.rgb = (0,1,0)
            else:
                self.box.rgb = (1,0,0)
            self.box.xy = map.to_pixel(mpos)
            self.box.render()

        elif self.current_tool == "path" and self.current_path:
            self.current_path.points.append(pos)
            self.current_path.build()
            self.current_path.render()
            self.current_path.points.pop()


        elif self.current_tool == "path":
            for p in self.scene.paths:
                #boxes
                for i in p.points:
                    self.path_node.xy = i
                    self.path_node.render()
                # center line
                glColor4f(0,0,1,.2)
                glBegin(GL_LINE_STRIP)
                for i in p.points:
                    glVertex2f(i[0], i[1])
                glEnd()


        elif self.current_tool == "script_area":
            for sa in self.script_areas.values():
                sa.render()

        if self.current_tool == "buildings":
            map = self.scene.map
            pos = map.from_pixel(pos)
            pos = map.to_pixel(pos)

            sx,sy = BUILDINGS[self.place_type].size
            box = rabbyt.Sprite(shape=(sx*32, sy*32), rgba=(1,1,1,.5))
            box.left = pos[0]
            box.bottom = pos[1]
            box.render()



class DummyGameScene(GameScene):
    def __init__(self, scene_handler):
        self.scene_handler = scene_handler
        self.ctrl = DummyCtrl(self)
        Scene.__init__(self)

    def load_map(self, mapname):
        self.map = MapWrapper(self, mapname)
        self.map.load()
        for o in self.map.display.static_map_objects_renderable:
            node = self.sprite_sets_staticmo.get(*o.xy)
            node.add(o)

    def loop(self):
        self.view.keep_within([0, self.map.size[1]*self.map.tile_size,
                self.map.size[0]*self.map.tile_size, 0])

        Scene.loop(self)
        rabbyt.clear((0,0,0,0))

        self.map.render()

        to_render = []
        sets = self.sprite_sets_staticmo.range(
                self.view.x-self.view.scale_w/2-32,
                self.view.y-self.view.scale_h/2-180,
                self.view.x+self.view.scale_w/2+32,
                self.view.y+self.view.scale_h/2)
        for s in sets:
            to_render.extend(s)
        sets = self.sprite_sets.range(
                self.view.x-self.view.scale_w/2-32,
                self.view.y-self.view.scale_h/2-180,
                self.view.x+self.view.scale_w/2+32,
                self.view.y+self.view.scale_h/2)
        for s in sets:
            to_render.extend(s)
        to_render.sort(key=lambda s: -s.y)

        rabbyt.render_unsorted(to_render)


        self.ctrl.update()
        self.ctrl.draw_m()
        self.ctrl.draw_gui()

        self.scene_handler.window.flip()


TYPES = MapModel.TYPES
sh = SceneHandler()
g = DummyGameScene(sh)

mapname = "crystalcrises"
if len(sys.argv) >= 2:
    mapname = sys.argv[1]

g.setup(mapname)
g.ctrl.setup()


while True:
    g.loop()
